@@ -1,10 +1,16 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package checks
 
 import (
+	"context"
 	"fmt"
+	"strings"
 	"sync"
 	"time"
 
+	"github.com/hashicorp/consul/acl"
 	"github.com/hashicorp/consul/agent/structs"
 	"github.com/hashicorp/consul/api"
 )
@@ -34,7 +40,7 @@ type CheckAlias struct {
 	stopLock sync.Mutex
 	stopWg   sync.WaitGroup
 
-	structs.EnterpriseMeta
+	acl.EnterpriseMeta
 }
 
 // AliasNotifier is a CheckNotifier specifically for the Alias check.
@@ -45,7 +51,7 @@ type AliasNotifier interface {
 
 	AddAliasCheck(structs.CheckID, structs.ServiceID, chan<- struct{}) error
 	RemoveAliasCheck(structs.CheckID, structs.ServiceID)
-	Checks(*structs.EnterpriseMeta) map[structs.CheckID]*structs.HealthCheck
+	Checks(*acl.EnterpriseMeta) map[structs.CheckID]*structs.HealthCheck
 }
 
 // Start is used to start the check, runs until Stop() func (c *CheckAlias) Start() {
@@ -108,7 +114,7 @@ func (c *CheckAlias) runLocal(stopCh chan struct{}) {
 	}
 
 	updateStatus := func() {
-		checks := c.Notify.Checks(structs.WildcardEnterpriseMeta())
+		checks := c.Notify.Checks(c.WithWildcardNamespace())
 		checksList := make([]*structs.HealthCheck, 0, len(checks))
 		for _, chk := range checks {
 			checksList = append(checksList, chk)
@@ -148,7 +154,7 @@ func (c *CheckAlias) checkServiceExistsOnRemoteServer(serviceID *structs.Service
 RETRY_CALL:
 	var out structs.IndexedNodeServices
 	attempts++
-	if err := c.RPC.RPC("Catalog.NodeServices", &args, &out); err != nil {
+	if err := c.RPC.RPC(context.Background(), "Catalog.NodeServices", &args, &out); err != nil {
 		if attempts <= 3 {
 			time.Sleep(time.Duration(attempts) * time.Second)
 			goto RETRY_CALL
@@ -205,7 +211,7 @@ func (c *CheckAlias) runQuery(stopCh chan struct{}) {
 		// index is global to the cluster.
 		var out structs.IndexedHealthChecks
 
-		if err := c.RPC.RPC("Health.NodeChecks", &args, &out); err != nil {
+		if err := c.RPC.RPC(context.Background(), "Health.NodeChecks", &args, &out); err != nil {
 			attempt++
 			if attempt > 1 {
 				c.Notify.UpdateCheck(c.CheckID, api.HealthCritical,
@@ -246,7 +252,7 @@ func (c *CheckAlias) processChecks(checks []*structs.HealthCheck, CheckIfService
 	msg := "No checks found."
 	serviceFound := false
 	for _, chk := range checks {
-		if c.Node != "" && c.Node != chk.Node {
+		if c.Node != "" && !strings.EqualFold(c.Node, chk.Node) {
 			continue
 		}
 		serviceMatch := c.ServiceID.Matches(chk.CompoundServiceID())
