@@ -1,6 +1,10 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package consul
 
 import (
+	"context"
 	"os"
 	"strings"
 	"testing"
@@ -9,7 +13,7 @@ import (
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 
-	"github.com/hashicorp/consul/agent/consul/state"
+	"github.com/hashicorp/consul/agent/configentry"
 	"github.com/hashicorp/consul/agent/structs"
 	tokenStore "github.com/hashicorp/consul/agent/token"
 	"github.com/hashicorp/consul/sdk/testutil/retry"
@@ -27,10 +31,10 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 
 	dir1, s1 := testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = "dc1"
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLMasterToken = "root"
-		c.ACLDefaultPolicy = "deny"
+		c.ACLInitialManagementToken = "root"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 		c.Build = "1.6.0"
 		c.OverrideInitialSerfTags = func(tags map[string]string) {
 			tags["ft_si"] = "0"
@@ -62,9 +66,9 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 	// dc2 as a secondary DC
 	dir2, s2 := testServerWithConfig(t, func(c *Config) {
 		c.Datacenter = "dc2"
-		c.ACLDatacenter = "dc1"
+		c.PrimaryDatacenter = "dc1"
 		c.ACLsEnabled = true
-		c.ACLDefaultPolicy = "deny"
+		c.ACLResolverSettings.ACLDefaultPolicy = "deny"
 		c.ACLTokenReplication = false
 		c.Build = "1.6.0"
 		c.OverrideInitialSerfTags = func(tags map[string]string) {
@@ -140,7 +144,7 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 			IntentionID:  ixn.Intention.ID,
 		}
 		var resp structs.IndexedIntentions
-		require.NoError(r, s2.RPC("Intention.Get", req, &resp), "ID=%q", ixn.Intention.ID)
+		require.NoError(r, s2.RPC(context.Background(), "Intention.Get", req, &resp), "ID=%q", ixn.Intention.ID)
 		require.Len(r, resp.Intentions, 1)
 
 		actual := resp.Intentions[0]
@@ -171,7 +175,7 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 			IntentionID:  ixn.Intention.ID,
 		}
 
-		require.NoError(r, s2.RPC("Intention.Get", req, &resp), "ID=%q", ixn.Intention.ID)
+		require.NoError(r, s2.RPC(context.Background(), "Intention.Get", req, &resp), "ID=%q", ixn.Intention.ID)
 		require.Len(r, resp.Intentions, 1)
 		require.Equal(r, "*", resp.Intentions[0].SourceName)
 	})
@@ -205,7 +209,7 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 			IntentionID:  ixn.Intention.ID,
 		}
 		var resp structs.IndexedIntentions
-		err := s2.RPC("Intention.Get", req, &resp)
+		err := s2.RPC(context.Background(), "Intention.Get", req, &resp)
 		require.Error(r, err)
 		if !strings.Contains(err.Error(), ErrIntentionNotFound.Error()) {
 			r.Fatalf("expected intention not found, got: %v", err)
@@ -217,7 +221,6 @@ func TestLeader_ReplicateIntentions(t *testing.T) {
 func TestLeader_batchLegacyIntentionUpdates(t *testing.T) {
 	t.Parallel()
 
-	assert := assert.New(t)
 	ixn1 := structs.TestIntention(t)
 	ixn1.ID = "ixn1"
 	ixn2 := structs.TestIntention(t)
@@ -356,7 +359,7 @@ func TestLeader_batchLegacyIntentionUpdates(t *testing.T) {
 
 	for _, tc := range cases {
 		actual := batchLegacyIntentionUpdates(tc.deletes, tc.updates)
-		assert.Equal(tc.expected, actual)
+		assert.Equal(t, tc.expected, actual)
 	}
 }
 
@@ -457,7 +460,7 @@ func TestLeader_LegacyIntentionMigration(t *testing.T) {
 
 	checkIntentions := func(t *testing.T, srv *Server, legacyOnly bool, expect map[string]*structs.Intention) {
 		t.Helper()
-		wildMeta := structs.WildcardEnterpriseMeta()
+		wildMeta := structs.WildcardEnterpriseMetaInDefaultPartition()
 		retry.Run(t, func(r *retry.R) {
 			var (
 				got structs.Intentions
@@ -520,7 +523,7 @@ func TestLeader_LegacyIntentionMigration(t *testing.T) {
 
 	// Wait until the migration routine is complete.
 	retry.Run(t, func(r *retry.R) {
-		intentionFormat, err := s1.getSystemMetadata(structs.SystemMetadataIntentionFormatKey)
+		intentionFormat, err := s1.GetSystemMetadata(structs.SystemMetadataIntentionFormatKey)
 		require.NoError(r, err)
 		if intentionFormat != structs.SystemMetadataIntentionFormatConfigValue {
 			r.Fatal("intention migration is not yet complete")
@@ -536,17 +539,17 @@ func TestLeader_LegacyIntentionMigration(t *testing.T) {
 		checkIntentions(t, s1, true, map[string]*structs.Intention{})
 	}))
 
-	mapifyConfigs := func(entries interface{}) map[state.ConfigEntryKindName]*structs.ServiceIntentionsConfigEntry {
-		m := make(map[state.ConfigEntryKindName]*structs.ServiceIntentionsConfigEntry)
+	mapifyConfigs := func(entries interface{}) map[configentry.KindName]*structs.ServiceIntentionsConfigEntry {
+		m := make(map[configentry.KindName]*structs.ServiceIntentionsConfigEntry)
 		switch v := entries.(type) {
 		case []*structs.ServiceIntentionsConfigEntry:
 			for _, entry := range v {
-				kn := state.NewConfigEntryKindName(entry.Kind, entry.Name, &entry.EnterpriseMeta)
+				kn := configentry.NewKindName(entry.Kind, entry.Name, &entry.EnterpriseMeta)
 				m[kn] = entry
 			}
 		case []structs.ConfigEntry:
 			for _, entry := range v {
-				kn := state.NewConfigEntryKindName(entry.GetKind(), entry.GetName(), entry.GetEnterpriseMeta())
+				kn := configentry.NewKindName(entry.GetKind(), entry.GetName(), entry.GetEnterpriseMeta())
 				m[kn] = entry.(*structs.ServiceIntentionsConfigEntry)
 			}
 		default:
@@ -556,7 +559,7 @@ func TestLeader_LegacyIntentionMigration(t *testing.T) {
 	}
 
 	// also check config entries
-	_, gotConfigs, err := s1.fsm.State().ConfigEntriesByKind(nil, structs.ServiceIntentions, structs.WildcardEnterpriseMeta())
+	_, gotConfigs, err := s1.fsm.State().ConfigEntriesByKind(nil, structs.ServiceIntentions, structs.WildcardEnterpriseMetaInDefaultPartition())
 	require.NoError(t, err)
 	gotConfigsM := mapifyConfigs(gotConfigs)
 
