@@ -1,8 +1,12 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package structs
 
 import (
 	"encoding/json"
 	"fmt"
+	"math/rand"
 	"reflect"
 	"strings"
 	"testing"
@@ -64,7 +68,6 @@ func TestStructs_Implements(t *testing.T) {
 		_ RPCInfo          = &SessionRequest{}
 		_ RPCInfo          = &SessionSpecificRequest{}
 		_ RPCInfo          = &EventFireRequest{}
-		_ RPCInfo          = &ACLPolicyResolveLegacyRequest{}
 		_ RPCInfo          = &ACLPolicyBatchGetRequest{}
 		_ RPCInfo          = &ACLPolicyGetRequest{}
 		_ RPCInfo          = &ACLTokenGetRequest{}
@@ -74,16 +77,6 @@ func TestStructs_Implements(t *testing.T) {
 }
 
 func TestStructs_RegisterRequest_ChangesNode(t *testing.T) {
-	req := &RegisterRequest{
-		ID:              types.NodeID("40e4a748-2192-161a-0510-9bf59fe950b5"),
-		Node:            "test",
-		Address:         "127.0.0.1",
-		Datacenter:      "dc1",
-		TaggedAddresses: make(map[string]string),
-		NodeMeta: map[string]string{
-			"role": "server",
-		},
-	}
 
 	node := &Node{
 		ID:              types.NodeID("40e4a748-2192-161a-0510-9bf59fe950b5"),
@@ -96,41 +89,104 @@ func TestStructs_RegisterRequest_ChangesNode(t *testing.T) {
 		},
 	}
 
-	check := func(twiddle, restore func()) {
-		if req.ChangesNode(node) {
-			t.Fatalf("should not change")
-		}
-
-		twiddle()
-		if !req.ChangesNode(node) {
-			t.Fatalf("should change")
-		}
-
-		req.SkipNodeUpdate = true
-		if req.ChangesNode(node) {
-			t.Fatalf("should skip")
-		}
-
-		req.SkipNodeUpdate = false
-		if !req.ChangesNode(node) {
-			t.Fatalf("should change")
-		}
-
-		restore()
-		if req.ChangesNode(node) {
-			t.Fatalf("should not change")
-		}
+	type testcase struct {
+		name   string
+		setup  func(*RegisterRequest)
+		expect bool
 	}
 
-	check(func() { req.ID = "nope" }, func() { req.ID = types.NodeID("40e4a748-2192-161a-0510-9bf59fe950b5") })
-	check(func() { req.Node = "nope" }, func() { req.Node = "test" })
-	check(func() { req.Address = "127.0.0.2" }, func() { req.Address = "127.0.0.1" })
-	check(func() { req.Datacenter = "dc2" }, func() { req.Datacenter = "dc1" })
-	check(func() { req.TaggedAddresses["wan"] = "nope" }, func() { delete(req.TaggedAddresses, "wan") })
-	check(func() { req.NodeMeta["invalid"] = "nope" }, func() { delete(req.NodeMeta, "invalid") })
+	cases := []testcase{
+		{
+			name: "id",
+			setup: func(r *RegisterRequest) {
+				r.ID = "nope"
+			},
+			expect: true,
+		},
+		{
+			name: "name",
+			setup: func(r *RegisterRequest) {
+				r.Node = "nope"
+			},
+			expect: true,
+		},
+		{
+			name: "name casing",
+			setup: func(r *RegisterRequest) {
+				r.Node = "TeSt"
+			},
+			expect: false,
+		},
+		{
+			name: "address",
+			setup: func(r *RegisterRequest) {
+				r.Address = "127.0.0.2"
+			},
+			expect: true,
+		},
+		{
+			name: "dc",
+			setup: func(r *RegisterRequest) {
+				r.Datacenter = "dc2"
+			},
+			expect: true,
+		},
+		{
+			name: "tagged addresses",
+			setup: func(r *RegisterRequest) {
+				r.TaggedAddresses["wan"] = "nope"
+			},
+			expect: true,
+		},
+		{
+			name: "node meta",
+			setup: func(r *RegisterRequest) {
+				r.NodeMeta["invalid"] = "nope"
+			},
+			expect: true,
+		},
+	}
 
-	if !req.ChangesNode(nil) {
-		t.Fatalf("should change")
+	run := func(t *testing.T, tc testcase) {
+		req := &RegisterRequest{
+			ID:              types.NodeID("40e4a748-2192-161a-0510-9bf59fe950b5"),
+			Node:            "test",
+			Address:         "127.0.0.1",
+			Datacenter:      "dc1",
+			TaggedAddresses: make(map[string]string),
+			NodeMeta: map[string]string{
+				"role": "server",
+			},
+		}
+
+		if req.ChangesNode(node) {
+			t.Fatalf("should not change")
+		}
+
+		tc.setup(req)
+
+		if tc.expect {
+			if !req.ChangesNode(node) {
+				t.Fatalf("should change")
+			}
+		} else {
+			if req.ChangesNode(node) {
+				t.Fatalf("should not change")
+			}
+		}
+
+		t.Run("skip node update", func(t *testing.T) {
+			req.SkipNodeUpdate = true
+			if req.ChangesNode(node) {
+				t.Fatalf("should skip")
+			}
+		})
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
+		})
 	}
 }
 
@@ -235,98 +291,228 @@ func TestNode_IsSame(t *testing.T) {
 			ModifyIndex: 2,
 		},
 	}
-	other := &Node{
-		ID:              id,
-		Node:            node,
-		Datacenter:      datacenter,
-		Address:         address,
-		TaggedAddresses: make(map[string]string),
-		Meta:            make(map[string]string),
-		RaftIndex: RaftIndex{
-			CreateIndex: 1,
-			ModifyIndex: 3,
+
+	type testcase struct {
+		name   string
+		setup  func(*Node)
+		expect bool
+	}
+	cases := []testcase{
+		{
+			name: "id",
+			setup: func(n *Node) {
+				n.ID = types.NodeID("")
+			},
+			expect: false,
+		},
+		{
+			name: "node",
+			setup: func(n *Node) {
+				n.Node = "other"
+			},
+			expect: false,
+		},
+		{
+			name: "node casing",
+			setup: func(n *Node) {
+				n.Node = "MyNoDe1"
+			},
+			expect: true,
+		},
+		{
+			name: "dc",
+			setup: func(n *Node) {
+				n.Datacenter = "dcX"
+			},
+			expect: false,
+		},
+		{
+			name: "address",
+			setup: func(n *Node) {
+				n.Address = "127.0.0.1"
+			},
+			expect: false,
+		},
+		{
+			name: "tagged addresses",
+			setup: func(n *Node) {
+				n.TaggedAddresses = map[string]string{"my": "address"}
+			},
+			expect: false,
+		},
+		{
+			name: "meta",
+			setup: func(n *Node) {
+				n.Meta = map[string]string{"my": "meta"}
+			},
+			expect: false,
 		},
 	}
-	check := func(twiddle, restore func()) {
-		t.Helper()
+
+	run := func(t *testing.T, tc testcase) {
+		other := &Node{
+			ID:              id,
+			Node:            node,
+			Datacenter:      datacenter,
+			Address:         address,
+			TaggedAddresses: make(map[string]string),
+			Meta:            make(map[string]string),
+			RaftIndex: RaftIndex{
+				CreateIndex: 1,
+				ModifyIndex: 3,
+			},
+		}
+
 		if !n.IsSame(other) || !other.IsSame(n) {
 			t.Fatalf("should be the same")
 		}
 
-		twiddle()
-		if n.IsSame(other) || other.IsSame(n) {
-			t.Fatalf("should be different, was %#v VS %#v", n, other)
-		}
+		tc.setup(other)
 
-		restore()
-		if !n.IsSame(other) || !other.IsSame(n) {
-			t.Fatalf("should be the same")
+		if tc.expect {
+			if !n.IsSame(other) || !other.IsSame(n) {
+				t.Fatalf("should be the same")
+			}
+		} else {
+			if n.IsSame(other) || other.IsSame(n) {
+				t.Fatalf("should be different, was %#v VS %#v", n, other)
+			}
 		}
 	}
-	check(func() { other.ID = types.NodeID("") }, func() { other.ID = id })
-	check(func() { other.Node = "other" }, func() { other.Node = node })
-	check(func() { other.Datacenter = "dcX" }, func() { other.Datacenter = datacenter })
-	check(func() { other.Address = "127.0.0.1" }, func() { other.Address = address })
-	check(func() { other.TaggedAddresses = map[string]string{"my": "address"} }, func() { other.TaggedAddresses = map[string]string{} })
-	check(func() { other.Meta = map[string]string{"my": "meta"} }, func() { other.Meta = map[string]string{} })
 
-	if !n.IsSame(other) {
-		t.Fatalf("should be equal, was %#v VS %#v", n, other)
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
+		})
 	}
 }
 
 func TestStructs_ServiceNode_IsSameService(t *testing.T) {
-	sn := testServiceNode(t)
-	node := "node1"
-	serviceID := sn.ServiceID
-	serviceAddress := sn.ServiceAddress
-	serviceEnableTagOverride := sn.ServiceEnableTagOverride
-	serviceMeta := make(map[string]string)
-	for k, v := range sn.ServiceMeta {
-		serviceMeta[k] = v
+	const (
+		nodeName = "node1"
+	)
+
+	type testcase struct {
+		name   string
+		setup  func(*ServiceNode)
+		expect bool
+	}
+	cases := []testcase{
+		{
+			name: "ServiceID",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceID = "66fb695a-c782-472f-8d36-4f3edd754b37"
+			},
+		},
+		{
+			name: "Node",
+			setup: func(sn *ServiceNode) {
+				sn.Node = "other"
+			},
+		},
+		{
+			name: "Node casing",
+			setup: func(sn *ServiceNode) {
+				sn.Node = "NoDe1"
+			},
+			expect: true,
+		},
+		{
+			name: "ServiceAddress",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceAddress = "1.2.3.4"
+			},
+		},
+		{
+			name: "ServiceEnableTagOverride",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceEnableTagOverride = !sn.ServiceEnableTagOverride
+			},
+		},
+		{
+			name: "ServiceKind",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceKind = "newKind"
+			},
+		},
+		{
+			name: "ServiceMeta",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceMeta = map[string]string{"my": "meta"}
+			},
+		},
+		{
+			name: "ServiceName",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceName = "duck"
+			},
+		},
+		{
+			name: "ServicePort",
+			setup: func(sn *ServiceNode) {
+				sn.ServicePort = 65534
+			},
+		},
+		{
+			name: "ServiceTags",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceTags = []string{"new", "tags"}
+			},
+		},
+		{
+			name: "ServiceWeights",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceWeights = Weights{Passing: 42, Warning: 41}
+			},
+		},
+		{
+			name: "ServiceProxy",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceProxy = ConnectProxyConfig{}
+			},
+		},
+		{
+			name: "ServiceConnect",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceConnect = ServiceConnect{}
+			},
+		},
+		{
+			name: "ServiceTaggedAddresses",
+			setup: func(sn *ServiceNode) {
+				sn.ServiceTaggedAddresses = nil
+			},
+		},
 	}
-	serviceName := sn.ServiceName
-	servicePort := sn.ServicePort
-	serviceTags := sn.ServiceTags
-	serviceWeights := Weights{Passing: 2, Warning: 1}
-	sn.ServiceWeights = serviceWeights
-	serviceProxy := sn.ServiceProxy
-	serviceConnect := sn.ServiceConnect
-	serviceTaggedAddresses := sn.ServiceTaggedAddresses
 
-	n := sn.ToNodeService().ToServiceNode(node)
-	other := sn.ToNodeService().ToServiceNode(node)
+	run := func(t *testing.T, tc testcase) {
+		sn := testServiceNode(t)
+		sn.ServiceWeights = Weights{Passing: 2, Warning: 1}
+		n := sn.ToNodeService().ToServiceNode(nodeName)
+		other := sn.ToNodeService().ToServiceNode(nodeName)
 
-	check := func(twiddle, restore func()) {
-		t.Helper()
 		if !n.IsSameService(other) || !other.IsSameService(n) {
 			t.Fatalf("should be the same")
 		}
 
-		twiddle()
-		if n.IsSameService(other) || other.IsSameService(n) {
-			t.Fatalf("should be different, was %#v VS %#v", n, other)
-		}
+		tc.setup(other)
 
-		restore()
-		if !n.IsSameService(other) || !other.IsSameService(n) {
-			t.Fatalf("should be the same after restore, was:\n %#v VS\n %#v", n, other)
+		if tc.expect {
+			if !n.IsSameService(other) || !other.IsSameService(n) {
+				t.Fatalf("should be the same")
+			}
+		} else {
+			if n.IsSameService(other) || other.IsSameService(n) {
+				t.Fatalf("should be different, was %#v VS %#v", n, other)
+			}
 		}
 	}
 
-	check(func() { other.ServiceID = "66fb695a-c782-472f-8d36-4f3edd754b37" }, func() { other.ServiceID = serviceID })
-	check(func() { other.Node = "other" }, func() { other.Node = node })
-	check(func() { other.ServiceAddress = "1.2.3.4" }, func() { other.ServiceAddress = serviceAddress })
-	check(func() { other.ServiceEnableTagOverride = !serviceEnableTagOverride }, func() { other.ServiceEnableTagOverride = serviceEnableTagOverride })
-	check(func() { other.ServiceKind = "newKind" }, func() { other.ServiceKind = "" })
-	check(func() { other.ServiceMeta = map[string]string{"my": "meta"} }, func() { other.ServiceMeta = serviceMeta })
-	check(func() { other.ServiceName = "duck" }, func() { other.ServiceName = serviceName })
-	check(func() { other.ServicePort = 65534 }, func() { other.ServicePort = servicePort })
-	check(func() { other.ServiceTags = []string{"new", "tags"} }, func() { other.ServiceTags = serviceTags })
-	check(func() { other.ServiceWeights = Weights{Passing: 42, Warning: 41} }, func() { other.ServiceWeights = serviceWeights })
-	check(func() { other.ServiceProxy = ConnectProxyConfig{} }, func() { other.ServiceProxy = serviceProxy })
-	check(func() { other.ServiceConnect = ServiceConnect{} }, func() { other.ServiceConnect = serviceConnect })
-	check(func() { other.ServiceTaggedAddresses = nil }, func() { other.ServiceTaggedAddresses = serviceTaggedAddresses })
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
+		})
+	}
 }
 
 func TestStructs_ServiceNode_PartialClone(t *testing.T) {
@@ -406,6 +592,43 @@ func TestStructs_ServiceNode_Conversions(t *testing.T) {
 	}
 }
 
+func TestStructs_Locality_Validate(t *testing.T) {
+	type testCase struct {
+		locality *Locality
+		err      string
+	}
+	cases := map[string]testCase{
+		"nil": {
+			nil,
+			"",
+		},
+		"region only": {
+			&Locality{Region: "us-west-1"},
+			"",
+		},
+		"region and zone": {
+			&Locality{Region: "us-west-1", Zone: "us-west-1a"},
+			"",
+		},
+		"zone only": {
+			&Locality{Zone: "us-west-1a"},
+			"zone cannot be set without region",
+		},
+	}
+
+	for name, tc := range cases {
+		t.Run(name, func(t *testing.T) {
+			err := tc.locality.Validate()
+			if tc.err == "" {
+				require.NoError(t, err)
+			} else {
+				require.Error(t, err)
+				require.Contains(t, err.Error(), tc.err)
+			}
+		})
+	}
+}
+
 func TestStructs_NodeService_ValidateMeshGateway(t *testing.T) {
 	type testCase struct {
 		Modify func(*NodeService)
@@ -496,6 +719,10 @@ func TestStructs_NodeService_ValidateTerminatingGateway(t *testing.T) {
 			func(x *NodeService) { x.Proxy.Upstreams = []Upstream{{}} },
 			"Proxy.Upstreams configuration is invalid",
 		},
+		"port": {
+			func(x *NodeService) { x.Port = 0 },
+			"Port must be non-zero",
+		},
 	}
 
 	for name, tc := range cases {
@@ -636,6 +863,16 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			"",
 		},
 
+		{
+			"connect-proxy: invalid opaque config",
+			func(x *NodeService) {
+				x.Proxy.Config = map[string]interface{}{
+					"envoy_hcp_metrics_bind_socket_dir": "/Consul/is/a/networking/platform/that/enables/securing/your/networking/",
+				}
+			},
+			"Proxy.Config: envoy_hcp_metrics_bind_socket_dir length 71 exceeds max",
+		},
+
 		{
 			"connect-proxy: no Proxy.DestinationServiceName",
 			func(x *NodeService) { x.Proxy.DestinationServiceName = "" },
@@ -663,7 +900,7 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 		{
 			"connect-proxy: no port set",
 			func(x *NodeService) { x.Port = 0 },
-			"port or socketpath must",
+			fmt.Sprintf("Port or SocketPath must be set for a %s", ServiceKindConnectProxy),
 		},
 
 		{
@@ -790,6 +1027,21 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			},
 			"",
 		},
+		{
+			"connect-proxy: Upstreams non default partition another dc",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{ // baseline
+						DestinationType:      UpstreamDestTypeService,
+						DestinationName:      "foo",
+						DestinationPartition: "foo",
+						Datacenter:           "dc1",
+						LocalBindPort:        5000,
+					},
+				}
+			},
+			"upstreams cannot target another datacenter in non default partition",
+		},
 		{
 			"connect-proxy: Upstreams duplicated by port",
 			func(x *NodeService) {
@@ -923,21 +1175,139 @@ func TestStructs_NodeService_ValidateConnectProxy(t *testing.T) {
 			},
 			"upstreams cannot contain duplicates",
 		},
+		{
+			"connect-proxy: valid Upstream.PeerDestination",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{
+						DestinationType: UpstreamDestTypeService,
+						DestinationName: "foo",
+						DestinationPeer: "peer1",
+						LocalBindPort:   5000,
+					},
+				}
+			},
+			"",
+		},
+		{
+			"connect-proxy: invalid locality",
+			func(x *NodeService) {
+				x.Locality = &Locality{Zone: "bad"}
+			},
+			"zone cannot be set without region",
+		},
 	}
 
 	for _, tc := range cases {
 		t.Run(tc.Name, func(t *testing.T) {
-			assert := assert.New(t)
 			ns := TestNodeServiceProxy(t)
 			tc.Modify(ns)
 
 			err := ns.Validate()
-			assert.Equal(err != nil, tc.Err != "", err)
+			assert.Equal(t, err != nil, tc.Err != "", err)
 			if err == nil {
 				return
 			}
 
-			assert.Contains(strings.ToLower(err.Error()), strings.ToLower(tc.Err))
+			assert.Contains(t, strings.ToLower(err.Error()), strings.ToLower(tc.Err))
+		})
+	}
+}
+
+func TestStructs_NodeService_ValidateConnectProxyWithAgentAutoAssign(t *testing.T) {
+	t.Run("connect-proxy: no port set", func(t *testing.T) {
+		ns := TestNodeServiceProxy(t)
+		ns.Port = 0
+
+		err := ns.ValidateForAgent()
+		assert.NoError(t, err)
+	})
+}
+
+func TestStructs_NodeService_ValidateConnectProxy_In_Partition(t *testing.T) {
+	cases := []struct {
+		Name   string
+		Modify func(*NodeService)
+		Err    string
+	}{
+		{
+			"valid",
+			func(x *NodeService) {},
+			"",
+		},
+		{
+			"connect-proxy: Upstreams non default partition another dc",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{ // baseline
+						DestinationType:      UpstreamDestTypeService,
+						DestinationName:      "foo",
+						DestinationPartition: "foo",
+						Datacenter:           "dc1",
+						LocalBindPort:        5000,
+					},
+				}
+			},
+			"upstreams cannot target another datacenter in non default partition",
+		},
+		{
+			"connect-proxy: Upstreams non default partition same dc",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{ // baseline
+						DestinationType:      UpstreamDestTypeService,
+						DestinationName:      "foo",
+						DestinationPartition: "foo",
+						LocalBindPort:        5000,
+					},
+				}
+			},
+			"",
+		},
+		{
+			"connect-proxy: Upstream with peer targets partition different from NodeService",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{
+						DestinationType:      UpstreamDestTypeService,
+						DestinationName:      "foo",
+						DestinationPartition: "part1",
+						DestinationPeer:      "peer1",
+						LocalBindPort:        5000,
+					},
+				}
+			},
+			"upstreams must target peers in the same partition as the service",
+		},
+		{
+			"connect-proxy: Upstream with peer defaults to NodeService's peer",
+			func(x *NodeService) {
+				x.Proxy.Upstreams = Upstreams{
+					{
+						DestinationType: UpstreamDestTypeService,
+						DestinationName: "foo",
+						// No DestinationPartition here but we assert that it defaults to "bar" and not "default"
+						DestinationPeer: "peer1",
+						LocalBindPort:   5000,
+					},
+				}
+			},
+			"",
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.Name, func(t *testing.T) {
+			ns := TestNodeServiceProxyInPartition(t, "bar")
+			tc.Modify(ns)
+
+			err := ns.Validate()
+			assert.Equal(t, err != nil, tc.Err != "", err)
+			if err == nil {
+				return
+			}
+
+			assert.Contains(t, strings.ToLower(err.Error()), strings.ToLower(tc.Err))
 		})
 	}
 }
@@ -973,21 +1343,29 @@ func TestStructs_NodeService_ValidateSidecarService(t *testing.T) {
 
 	for _, tc := range cases {
 		t.Run(tc.Name, func(t *testing.T) {
-			assert := assert.New(t)
 			ns := TestNodeServiceSidecar(t)
 			tc.Modify(ns)
 
 			err := ns.Validate()
-			assert.Equal(err != nil, tc.Err != "", err)
+			assert.Equal(t, err != nil, tc.Err != "", err)
 			if err == nil {
 				return
 			}
 
-			assert.Contains(strings.ToLower(err.Error()), strings.ToLower(tc.Err))
+			assert.Contains(t, strings.ToLower(err.Error()), strings.ToLower(tc.Err))
 		})
 	}
 }
 
+func TestStructs_NodeService_ConnectNativeEmptyPortError(t *testing.T) {
+	ns := TestNodeService()
+	ns.Connect.Native = true
+	ns.Port = 0
+	err := ns.Validate()
+	assert.Error(t, err)
+	assert.Contains(t, err.Error(), "Port or SocketPath must be set for a Connect native service.")
+}
+
 func TestStructs_NodeService_IsSame(t *testing.T) {
 	ns := &NodeService{
 		ID:      "node1",
@@ -1105,82 +1483,125 @@ func TestStructs_NodeService_IsSame(t *testing.T) {
 }
 
 func TestStructs_HealthCheck_IsSame(t *testing.T) {
-	hc := &HealthCheck{
-		Node:        "node1",
-		CheckID:     "check1",
-		Name:        "thecheck",
-		Status:      api.HealthPassing,
-		Notes:       "it's all good",
-		Output:      "lgtm",
-		ServiceID:   "service1",
-		ServiceName: "theservice",
-		ServiceTags: []string{"foo"},
-	}
-	if !hc.IsSame(hc) {
-		t.Fatalf("should be equal to itself")
+	type testcase struct {
+		name   string
+		setup  func(*HealthCheck)
+		expect bool
 	}
 
-	other := &HealthCheck{
-		Node:        "node1",
-		CheckID:     "check1",
-		Name:        "thecheck",
-		Status:      api.HealthPassing,
-		Notes:       "it's all good",
-		Output:      "lgtm",
-		ServiceID:   "service1",
-		ServiceName: "theservice",
-		ServiceTags: []string{"foo"},
-		RaftIndex: RaftIndex{
-			CreateIndex: 1,
-			ModifyIndex: 2,
+	cases := []testcase{
+		{
+			name: "Node",
+			setup: func(hc *HealthCheck) {
+				hc.Node = "XXX"
+			},
+		},
+		{
+			name: "Node casing",
+			setup: func(hc *HealthCheck) {
+				hc.Node = "NoDe1"
+			},
+			expect: true,
+		},
+		{
+			name: "CheckID",
+			setup: func(hc *HealthCheck) {
+				hc.CheckID = "XXX"
+			},
+		},
+		{
+			name: "Name",
+			setup: func(hc *HealthCheck) {
+				hc.Name = "XXX"
+			},
+		},
+		{
+			name: "Status",
+			setup: func(hc *HealthCheck) {
+				hc.Status = "XXX"
+			},
+		},
+		{
+			name: "Notes",
+			setup: func(hc *HealthCheck) {
+				hc.Notes = "XXX"
+			},
+		},
+		{
+			name: "Output",
+			setup: func(hc *HealthCheck) {
+				hc.Output = "XXX"
+			},
+		},
+		{
+			name: "ServiceID",
+			setup: func(hc *HealthCheck) {
+				hc.ServiceID = "XXX"
+			},
+		},
+		{
+			name: "ServiceName",
+			setup: func(hc *HealthCheck) {
+				hc.ServiceName = "XXX"
+			},
 		},
 	}
-	if !hc.IsSame(other) || !other.IsSame(hc) {
-		t.Fatalf("should not care about Raft fields")
-	}
 
-	checkCheckIDField := func(field *types.CheckID) {
-		if !hc.IsSame(other) || !other.IsSame(hc) {
-			t.Fatalf("should be the same")
+	run := func(t *testing.T, tc testcase) {
+		hc := &HealthCheck{
+			Node:        "node1",
+			CheckID:     "check1",
+			Name:        "thecheck",
+			Status:      api.HealthPassing,
+			Notes:       "it's all good",
+			Output:      "lgtm",
+			ServiceID:   "service1",
+			ServiceName: "theservice",
+			ServiceTags: []string{"foo"},
 		}
 
-		old := *field
-		*field = "XXX"
-		if hc.IsSame(other) || other.IsSame(hc) {
-			t.Fatalf("should not be the same")
+		if !hc.IsSame(hc) {
+			t.Fatalf("should be equal to itself")
+		}
+
+		other := &HealthCheck{
+			Node:        "node1",
+			CheckID:     "check1",
+			Name:        "thecheck",
+			Status:      api.HealthPassing,
+			Notes:       "it's all good",
+			Output:      "lgtm",
+			ServiceID:   "service1",
+			ServiceName: "theservice",
+			ServiceTags: []string{"foo"},
+			RaftIndex: RaftIndex{
+				CreateIndex: 1,
+				ModifyIndex: 2,
+			},
 		}
-		*field = old
 
 		if !hc.IsSame(other) || !other.IsSame(hc) {
-			t.Fatalf("should be the same")
+			t.Fatalf("should not care about Raft fields")
+		}
+
+		tc.setup(hc)
+
+		if tc.expect {
+			if !hc.IsSame(other) || !other.IsSame(hc) {
+				t.Fatalf("should be the same")
+			}
+		} else {
+			if hc.IsSame(other) || other.IsSame(hc) {
+				t.Fatalf("should not be the same")
+			}
 		}
 	}
 
-	checkStringField := func(field *string) {
-		if !hc.IsSame(other) || !other.IsSame(hc) {
-			t.Fatalf("should be the same")
-		}
-
-		old := *field
-		*field = "XXX"
-		if hc.IsSame(other) || other.IsSame(hc) {
-			t.Fatalf("should not be the same")
-		}
-		*field = old
-
-		if !hc.IsSame(other) || !other.IsSame(hc) {
-			t.Fatalf("should be the same")
-		}
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
+		})
 	}
-
-	checkStringField(&other.Node)
-	checkCheckIDField(&other.CheckID)
-	checkStringField(&other.Name)
-	checkStringField(&other.Status)
-	checkStringField(&other.Notes)
-	checkStringField(&other.Output)
-	checkStringField(&other.ServiceID)
-	checkStringField(&other.ServiceName)
 }
 
 func TestStructs_HealthCheck_Marshalling(t *testing.T) {
@@ -1374,7 +1795,7 @@ func TestCheckServiceNode_CanRead(t *testing.T) {
 				Node:    &Node{Node: "name"},
 				Service: &NodeService{Service: "service-name"},
 			},
-			authz:    aclAuthorizerCheckServiceNode{allowService: true},
+			authz:    aclAuthorizerCheckServiceNode{allowLocalService: true},
 			expected: acl.Deny,
 		},
 		{
@@ -1383,7 +1804,7 @@ func TestCheckServiceNode_CanRead(t *testing.T) {
 				Node:    &Node{Node: "name"},
 				Service: &NodeService{Service: "service-name"},
 			},
-			authz:    aclAuthorizerCheckServiceNode{allowNode: true},
+			authz:    aclAuthorizerCheckServiceNode{allowLocalNode: true},
 			expected: acl.Deny,
 		},
 		{
@@ -1395,6 +1816,24 @@ func TestCheckServiceNode_CanRead(t *testing.T) {
 			authz:    acl.AllowAll(),
 			expected: acl.Allow,
 		},
+		{
+			name: "can read imported csn if can read imported data",
+			csn: CheckServiceNode{
+				Node:    &Node{Node: "name", PeerName: "cluster-2"},
+				Service: &NodeService{Service: "service-name", PeerName: "cluster-2"},
+			},
+			authz:    aclAuthorizerCheckServiceNode{allowImported: true},
+			expected: acl.Allow,
+		},
+		{
+			name: "can't read imported csn with authz for local services and nodes",
+			csn: CheckServiceNode{
+				Node:    &Node{Node: "name", PeerName: "cluster-2"},
+				Service: &NodeService{Service: "service-name", PeerName: "cluster-2"},
+			},
+			authz:    aclAuthorizerCheckServiceNode{allowLocalService: true, allowLocalNode: true},
+			expected: acl.Deny,
+		},
 	}
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
@@ -1405,19 +1844,34 @@ func TestCheckServiceNode_CanRead(t *testing.T) {
 
 type aclAuthorizerCheckServiceNode struct {
 	acl.Authorizer
-	allowNode    bool
-	allowService bool
+	allowLocalNode    bool
+	allowLocalService bool
+	allowImported     bool
 }
 
-func (a aclAuthorizerCheckServiceNode) ServiceRead(string, *acl.AuthorizerContext) acl.EnforcementDecision {
-	if a.allowService {
+func (a aclAuthorizerCheckServiceNode) ServiceRead(_ string, ctx *acl.AuthorizerContext) acl.EnforcementDecision {
+	if ctx.Peer != "" {
+		if a.allowImported {
+			return acl.Allow
+		}
+		return acl.Deny
+	}
+
+	if a.allowLocalService {
 		return acl.Allow
 	}
 	return acl.Deny
 }
 
-func (a aclAuthorizerCheckServiceNode) NodeRead(string, *acl.AuthorizerContext) acl.EnforcementDecision {
-	if a.allowNode {
+func (a aclAuthorizerCheckServiceNode) NodeRead(_ string, ctx *acl.AuthorizerContext) acl.EnforcementDecision {
+	if ctx.Peer != "" {
+		if a.allowImported {
+			return acl.Allow
+		}
+		return acl.Deny
+	}
+
+	if a.allowLocalNode {
 		return acl.Allow
 	}
 	return acl.Deny
@@ -1488,7 +1942,7 @@ func TestStructs_ValidateServiceAndNodeMetadata(t *testing.T) {
 		},
 		"reserved key prefix denied": {
 			map[string]string{
-				metaKeyReservedPrefix + "key": "value1",
+				MetaKeyReservedPrefix + "key": "value1",
 			},
 			false,
 			"reserved for internal use",
@@ -1497,7 +1951,7 @@ func TestStructs_ValidateServiceAndNodeMetadata(t *testing.T) {
 		},
 		"reserved key prefix allowed": {
 			map[string]string{
-				metaKeyReservedPrefix + "key": "value1",
+				MetaKeyReservedPrefix + "key": "value1",
 			},
 			true,
 			"",
@@ -1567,13 +2021,13 @@ func TestStructs_validateMetaPair(t *testing.T) {
 		// key too long
 		{longKey, "value", "Key is too long", false, nil},
 		// reserved prefix
-		{metaKeyReservedPrefix + "key", "value", "reserved for internal use", false, nil},
+		{MetaKeyReservedPrefix + "key", "value", "reserved for internal use", false, nil},
 		// reserved prefix, allowed
-		{metaKeyReservedPrefix + "key", "value", "", true, nil},
+		{MetaKeyReservedPrefix + "key", "value", "", true, nil},
 		// reserved prefix, not allowed via an allowlist
-		{metaKeyReservedPrefix + "bad", "value", "reserved for internal use", false, map[string]struct{}{metaKeyReservedPrefix + "good": {}}},
+		{MetaKeyReservedPrefix + "bad", "value", "reserved for internal use", false, map[string]struct{}{MetaKeyReservedPrefix + "good": {}}},
 		// reserved prefix, allowed via an allowlist
-		{metaKeyReservedPrefix + "good", "value", "", true, map[string]struct{}{metaKeyReservedPrefix + "good": {}}},
+		{MetaKeyReservedPrefix + "good", "value", "", true, map[string]struct{}{MetaKeyReservedPrefix + "good": {}}},
 		// value too long
 		{"key", longValue, "Value is too long", false, nil},
 	}
@@ -1612,6 +2066,8 @@ func TestServiceDumpRequest_CacheInfoKey(t *testing.T) {
 var cacheInfoIgnoredFields = map[string]bool{
 	// Datacenter is part of the cache key added by the cache itself.
 	"Datacenter": true,
+	// PeerName is part of the cache key added by the cache itself.
+	"PeerName": true,
 	// QuerySource is always the same for every request from a single agent, so it
 	// is excluded from the key.
 	"Source": true,
@@ -2035,14 +2491,18 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 		input   CheckServiceNode
 		lanAddr string
 		lanPort int
+		lanIdx  uint64
 		wanAddr string
 		wanPort int
+		wanIdx  uint64
 	}
 
 	nodeAddr := "10.1.2.3"
 	nodeWANAddr := "198.18.19.20"
+	nodeIdx := uint64(11)
 	serviceAddr := "10.2.3.4"
 	servicePort := 1234
+	serviceIdx := uint64(22)
 	serviceWANAddr := "198.19.20.21"
 	serviceWANPort := 987
 
@@ -2051,15 +2511,23 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 			input: CheckServiceNode{
 				Node: &Node{
 					Address: nodeAddr,
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: nodeAddr,
+			wanIdx:  nodeIdx,
 			wanPort: servicePort,
 		},
 		"node-wan-address": {
@@ -2069,15 +2537,23 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: nodeWANAddr,
+			wanIdx:  nodeIdx,
 			wanPort: servicePort,
 		},
 		"service-address": {
@@ -2088,16 +2564,24 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
 					Port:    servicePort,
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceAddr,
+			wanIdx:  serviceIdx,
 			wanPort: servicePort,
 		},
 		"service-wan-address": {
@@ -2108,6 +2592,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
@@ -2118,12 +2605,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    serviceWANPort,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: serviceWANPort,
 		},
 		"service-wan-address-default-port": {
@@ -2134,6 +2626,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Address: serviceAddr,
@@ -2144,12 +2639,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    0,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: serviceAddr,
+			lanIdx:  serviceIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: servicePort,
 		},
 		"service-wan-address-node-lan": {
@@ -2160,6 +2660,9 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 					TaggedAddresses: map[string]string{
 						"wan": nodeWANAddr,
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: nodeIdx,
+					},
 				},
 				Service: &NodeService{
 					Port: servicePort,
@@ -2169,12 +2672,17 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 							Port:    serviceWANPort,
 						},
 					},
+					RaftIndex: RaftIndex{
+						ModifyIndex: serviceIdx,
+					},
 				},
 			},
 
 			lanAddr: nodeAddr,
+			lanIdx:  nodeIdx,
 			lanPort: servicePort,
 			wanAddr: serviceWANAddr,
+			wanIdx:  serviceIdx,
 			wanPort: serviceWANPort,
 		},
 	}
@@ -2184,13 +2692,15 @@ func TestCheckServiceNode_BestAddress(t *testing.T) {
 		tc := tc
 		t.Run(name, func(t *testing.T) {
 
-			addr, port := tc.input.BestAddress(false)
+			idx, addr, port := tc.input.BestAddress(false)
 			require.Equal(t, tc.lanAddr, addr)
 			require.Equal(t, tc.lanPort, port)
+			require.Equal(t, tc.lanIdx, idx)
 
-			addr, port = tc.input.BestAddress(true)
+			idx, addr, port = tc.input.BestAddress(true)
 			require.Equal(t, tc.wanAddr, addr)
 			require.Equal(t, tc.wanPort, port)
+			require.Equal(t, tc.wanIdx, idx)
 		})
 	}
 }
@@ -2376,10 +2886,11 @@ func TestSnapshotRequestResponse_MsgpackEncodeDecode(t *testing.T) {
 		in := &SnapshotResponse{
 			Error: "blah",
 			QueryMeta: QueryMeta{
-				Index:            3,
-				LastContact:      5 * time.Second,
-				KnownLeader:      true,
-				ConsistencyLevel: "default",
+				Index:                 3,
+				LastContact:           5 * time.Second,
+				KnownLeader:           true,
+				ConsistencyLevel:      "default",
+				ResultsFilteredByACLs: true,
 			},
 		}
 		TestMsgpackEncodeDecode(t, in, true)
@@ -2446,3 +2957,82 @@ func TestGatewayService_IsSame(t *testing.T) {
 		t.Fatalf("should be equal, was %#v VS %#v", g, other)
 	}
 }
+
+func TestServiceList_Sort(t *testing.T) {
+	type testcase struct {
+		name   string
+		list   []ServiceName
+		expect []ServiceName
+	}
+
+	run := func(t *testing.T, tc testcase) {
+		t.Run("written order", func(t *testing.T) {
+			ServiceList(tc.list).Sort()
+			require.Equal(t, tc.expect, tc.list)
+		})
+		t.Run("random order", func(t *testing.T) {
+			rand.Shuffle(len(tc.list), func(i, j int) {
+				tc.list[i], tc.list[j] = tc.list[j], tc.list[i]
+			})
+			ServiceList(tc.list).Sort()
+			require.Equal(t, tc.expect, tc.list)
+		})
+	}
+
+	sn := func(name string) ServiceName {
+		return NewServiceName(name, nil)
+	}
+
+	cases := []testcase{
+		{
+			name:   "nil",
+			list:   nil,
+			expect: nil,
+		},
+		{
+			name:   "empty",
+			list:   []ServiceName{},
+			expect: []ServiceName{},
+		},
+		{
+			name:   "one",
+			list:   []ServiceName{sn("foo")},
+			expect: []ServiceName{sn("foo")},
+		},
+		{
+			name: "multiple",
+			list: []ServiceName{
+				sn("food"),
+				sn("zip"),
+				sn("Bar"),
+				sn("ba"),
+				sn("foo"),
+				sn("bar"),
+				sn("Foo"),
+				sn("Zip"),
+				sn("foo"),
+				sn("bar"),
+				sn("barrier"),
+			},
+			expect: []ServiceName{
+				sn("Bar"),
+				sn("Foo"),
+				sn("Zip"),
+				sn("ba"),
+				sn("bar"),
+				sn("bar"),
+				sn("barrier"),
+				sn("foo"),
+				sn("foo"),
+				sn("food"),
+				sn("zip"),
+			},
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.name, func(t *testing.T) {
+			run(t, tc)
+		})
+	}
+}
