@@ -1,34 +1,45 @@
+// Copyright (c) HashiCorp, Inc.
+// SPDX-License-Identifier: BUSL-1.1
+
 package xds
 
 import (
-	"github.com/hashicorp/consul/agent/connect"
 	"sort"
 	"sync"
 	"testing"
 	"time"
 
+	"github.com/armon/go-metrics"
 	envoy_cluster_v3 "github.com/envoyproxy/go-control-plane/envoy/config/cluster/v3"
 	envoy_core_v3 "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
 	envoy_endpoint_v3 "github.com/envoyproxy/go-control-plane/envoy/config/endpoint/v3"
 	envoy_listener_v3 "github.com/envoyproxy/go-control-plane/envoy/config/listener/v3"
 	envoy_rbac_v3 "github.com/envoyproxy/go-control-plane/envoy/config/rbac/v3"
 	envoy_route_v3 "github.com/envoyproxy/go-control-plane/envoy/config/route/v3"
+	envoy_http_router_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/router/v3"
 	envoy_http_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3"
 	envoy_network_rbac_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/rbac/v3"
 	envoy_tcp_proxy_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/tcp_proxy/v3"
 	envoy_tls_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/transport_sockets/tls/v3"
+	envoy_upstreams_v3 "github.com/envoyproxy/go-control-plane/envoy/extensions/upstreams/http/v3"
 	envoy_discovery_v3 "github.com/envoyproxy/go-control-plane/envoy/service/discovery/v3"
 	envoy_type_v3 "github.com/envoyproxy/go-control-plane/envoy/type/v3"
-
-	"github.com/armon/go-metrics"
-	"github.com/golang/protobuf/proto"
-	"github.com/golang/protobuf/ptypes"
-	"github.com/golang/protobuf/ptypes/wrappers"
 	"github.com/mitchellh/copystructure"
 	"github.com/stretchr/testify/require"
+	"google.golang.org/protobuf/proto"
+	"google.golang.org/protobuf/types/known/anypb"
+	"google.golang.org/protobuf/types/known/durationpb"
+	"google.golang.org/protobuf/types/known/wrapperspb"
 
+	"github.com/hashicorp/consul/agent/connect"
+	"github.com/hashicorp/consul/agent/grpc-external/limiter"
 	"github.com/hashicorp/consul/agent/proxycfg"
+	"github.com/hashicorp/consul/agent/proxycfg-sources/catalog"
 	"github.com/hashicorp/consul/agent/structs"
+	"github.com/hashicorp/consul/agent/xds/response"
+	"github.com/hashicorp/consul/envoyextensions/xdscommon"
+	proxysnapshot "github.com/hashicorp/consul/internal/mesh/proxy-snapshot"
+	"github.com/hashicorp/consul/proto-public/pbresource"
 	"github.com/hashicorp/consul/sdk/testutil"
 )
 
@@ -39,11 +50,12 @@ func newTestSnapshot(
 	t *testing.T,
 	prevSnap *proxycfg.ConfigSnapshot,
 	dbServiceProtocol string,
+	nsFn func(ns *structs.NodeService),
 	additionalEntries ...structs.ConfigEntry,
 ) *proxycfg.ConfigSnapshot {
-	snap := proxycfg.TestConfigSnapshotDiscoveryChainDefaultWithEntries(t, additionalEntries...)
-	snap.ConnectProxy.PreparedQueryEndpoints = map[string]structs.CheckServiceNodes{
-		"prepared_query:geo-cache": proxycfg.TestPreparedQueryNodes(t, "geo-cache"),
+	snap := proxycfg.TestConfigSnapshotDiscoveryChain(t, "default", false, nsFn, nil, additionalEntries...)
+	snap.ConnectProxy.PreparedQueryEndpoints = map[proxycfg.UpstreamID]structs.CheckServiceNodes{
+		UID("prepared_query:geo-cache"): proxycfg.TestPreparedQueryNodes(t, "geo-cache"),
 	}
 	if prevSnap != nil {
 		snap.Roots = prevSnap.Roots
@@ -52,7 +64,7 @@ func newTestSnapshot(
 	if dbServiceProtocol != "" {
 		// Simulate ServiceManager injection of protocol
 		snap.Proxy.Upstreams[0].Config["protocol"] = dbServiceProtocol
-		snap.ConnectProxy.ConfigSnapshotUpstreams.UpstreamConfig = snap.Proxy.Upstreams.ToMap()
+		snap.ConnectProxy.ConfigSnapshotUpstreams.UpstreamConfig = proxycfg.UpstreamsToMap(snap.Proxy.Upstreams)
 	}
 	return snap
 }
@@ -61,14 +73,16 @@ func newTestSnapshot(
 // testing. It also implements ConnectAuthz to allow control over authorization.
 type testManager struct {
 	sync.Mutex
-	chans   map[structs.ServiceID]chan *proxycfg.ConfigSnapshot
-	cancels chan structs.ServiceID
+	stateChans map[structs.ServiceID]chan proxysnapshot.ProxySnapshot
+	drainChans map[structs.ServiceID]chan struct{}
+	cancels    chan structs.ServiceID
 }
 
 func newTestManager(t *testing.T) *testManager {
 	return &testManager{
-		chans:   map[structs.ServiceID]chan *proxycfg.ConfigSnapshot{},
-		cancels: make(chan structs.ServiceID, 10),
+		stateChans: map[structs.ServiceID]chan proxysnapshot.ProxySnapshot{},
+		drainChans: map[structs.ServiceID]chan struct{}{},
+		cancels:    make(chan structs.ServiceID, 10),
 	}
 }
 
@@ -76,29 +90,57 @@ func newTestManager(t *testing.T) *testManager {
 func (m *testManager) RegisterProxy(t *testing.T, proxyID structs.ServiceID) {
 	m.Lock()
 	defer m.Unlock()
-	m.chans[proxyID] = make(chan *proxycfg.ConfigSnapshot, 1)
+	m.stateChans[proxyID] = make(chan proxysnapshot.ProxySnapshot, 1)
+	m.drainChans[proxyID] = make(chan struct{})
 }
 
 // Deliver simulates a proxy registration
-func (m *testManager) DeliverConfig(t *testing.T, proxyID structs.ServiceID, cfg *proxycfg.ConfigSnapshot) {
+func (m *testManager) DeliverConfig(t *testing.T, proxyID structs.ServiceID, cfg proxysnapshot.ProxySnapshot) {
 	t.Helper()
 	m.Lock()
 	defer m.Unlock()
 	select {
-	case m.chans[proxyID] <- cfg:
+	case m.stateChans[proxyID] <- cfg:
 	case <-time.After(10 * time.Millisecond):
 		t.Fatalf("took too long to deliver config")
 	}
 }
 
-// Watch implements ConfigManager
-func (m *testManager) Watch(proxyID structs.ServiceID) (<-chan *proxycfg.ConfigSnapshot, proxycfg.CancelFunc) {
+// DrainStreams drains any open streams for the given proxyID. If there aren't
+// any open streams, it'll create a marker so that future attempts to watch the
+// given proxyID will return limiter.ErrCapacityReached.
+func (m *testManager) DrainStreams(proxyID structs.ServiceID) {
 	m.Lock()
 	defer m.Unlock()
-	// ch might be nil but then it will just block forever
-	return m.chans[proxyID], func() {
-		m.cancels <- proxyID
+
+	ch, ok := m.drainChans[proxyID]
+	if !ok {
+		ch = make(chan struct{})
+		m.drainChans[proxyID] = ch
 	}
+	close(ch)
+}
+
+// Watch implements ConfigManager
+func (m *testManager) Watch(id *pbresource.ID, _ string, _ string) (<-chan proxysnapshot.ProxySnapshot,
+	limiter.SessionTerminatedChan, proxysnapshot.CancelFunc, error) {
+	// Create service ID
+	proxyID := structs.NewServiceID(id.Name, catalog.GetEnterpriseMetaFromResourceID(id))
+	m.Lock()
+	defer m.Unlock()
+
+	// If the drain chan has already been closed, return limiter.ErrCapacityReached.
+	drainCh := m.drainChans[proxyID]
+	select {
+	case <-drainCh:
+		return nil, nil, nil, limiter.ErrCapacityReached
+	default:
+	}
+
+	// ch might be nil but then it will just block forever
+	return m.stateChans[proxyID], drainCh, func() {
+		m.cancels <- proxyID
+	}, nil
 }
 
 // AssertWatchCancelled checks that the most recent call to a Watch cancel func
@@ -124,39 +166,15 @@ type testServerScenario struct {
 	errCh  <-chan error
 }
 
-func newTestServerScenario(
-	t *testing.T,
-	resolveToken ACLResolverFunc,
-	proxyID string,
-	token string,
-	authCheckFrequency time.Duration,
-) *testServerScenario {
-	return newTestServerScenarioInner(t, resolveToken, proxyID, token, authCheckFrequency, false)
-}
-
 func newTestServerDeltaScenario(
 	t *testing.T,
-	resolveToken ACLResolverFunc,
+	resolveTokenSecret ACLResolverFunc,
 	proxyID string,
 	token string,
 	authCheckFrequency time.Duration,
-) *testServerScenario {
-	return newTestServerScenarioInner(t, resolveToken, proxyID, token, authCheckFrequency, true)
-}
-
-func newTestServerScenarioInner(
-	t *testing.T,
-	resolveToken ACLResolverFunc,
-	proxyID string,
-	token string,
-	authCheckFrequency time.Duration,
-	incremental bool,
 ) *testServerScenario {
 	mgr := newTestManager(t)
 	envoy := NewTestEnvoy(t, proxyID, token)
-	t.Cleanup(func() {
-		envoy.Close()
-	})
 
 	sink := metrics.NewInmemSink(1*time.Minute, 1*time.Minute)
 	cfg := metrics.DefaultConfig("consul.xds.test")
@@ -165,15 +183,16 @@ func newTestServerScenarioInner(
 	metrics.NewGlobal(cfg, sink)
 
 	t.Cleanup(func() {
+		envoy.Close()
 		sink := &metrics.BlackholeSink{}
 		metrics.NewGlobal(cfg, sink)
 	})
 
 	s := NewServer(
+		"node-123",
 		testutil.Logger(t),
 		mgr,
-		resolveToken,
-		nil, /*checkFetcher HTTPCheckFetcher*/
+		resolveTokenSecret,
 		nil, /*cfgFetcher ConfigFetcher*/
 	)
 	if authCheckFrequency > 0 {
@@ -182,12 +201,7 @@ func newTestServerScenarioInner(
 
 	errCh := make(chan error, 1)
 	go func() {
-		if incremental {
-			errCh <- s.DeltaAggregatedResources(envoy.deltaStream)
-		} else {
-			shim := &adsServerV2Shim{srv: s}
-			errCh <- shim.StreamAggregatedResources(envoy.stream)
-		}
+		errCh <- s.DeltaAggregatedResources(envoy.deltaStream)
 	}()
 
 	return &testServerScenario{
@@ -219,7 +233,7 @@ func xdsNewEndpoint(ip string, port int) *envoy_endpoint_v3.LbEndpoint {
 	return &envoy_endpoint_v3.LbEndpoint{
 		HostIdentifier: &envoy_endpoint_v3.LbEndpoint_Endpoint{
 			Endpoint: &envoy_endpoint_v3.Endpoint{
-				Address: makeAddress(ip, port),
+				Address: response.MakeAddress(ip, port),
 			},
 		},
 	}
@@ -228,7 +242,7 @@ func xdsNewEndpoint(ip string, port int) *envoy_endpoint_v3.LbEndpoint {
 func xdsNewEndpointWithHealth(ip string, port int, health envoy_core_v3.HealthStatus, weight int) *envoy_endpoint_v3.LbEndpoint {
 	ep := xdsNewEndpoint(ip, port)
 	ep.HealthStatus = health
-	ep.LoadBalancingWeight = makeUint32Value(weight)
+	ep.LoadBalancingWeight = response.MakeUint32Value(weight)
 	return ep
 }
 
@@ -252,9 +266,9 @@ func xdsNewUpstreamTransportSocket(
 	t *testing.T,
 	snap *proxycfg.ConfigSnapshot,
 	sni string,
-	uri ...connect.SpiffeIDService,
+	spiffeID ...string,
 ) *envoy_core_v3.TransportSocket {
-	return xdsNewTransportSocket(t, snap, false, false, sni, uri...)
+	return xdsNewTransportSocket(t, snap, false, false, sni, spiffeID...)
 }
 
 func xdsNewTransportSocket(
@@ -263,7 +277,7 @@ func xdsNewTransportSocket(
 	downstream bool,
 	requireClientCert bool,
 	sni string,
-	uri ...connect.SpiffeIDService,
+	spiffeID ...string,
 ) *envoy_core_v3.TransportSocket {
 	// Assume just one root for now, can get fancier later if needed.
 	caPEM := snap.Roots.Roots[0].RootCert
@@ -280,15 +294,15 @@ func xdsNewTransportSocket(
 			},
 		},
 	}
-	if len(uri) > 0 {
-		require.NoError(t, injectSANMatcher(commonTLSContext, uri...))
+	if len(spiffeID) > 0 {
+		require.NoError(t, injectSANMatcher(commonTLSContext, false, spiffeID...))
 	}
 
 	var tlsContext proto.Message
 	if downstream {
-		var requireClientCertPB *wrappers.BoolValue
+		var requireClientCertPB *wrapperspb.BoolValue
 		if requireClientCert {
-			requireClientCertPB = makeBoolValue(true)
+			requireClientCertPB = response.MakeBoolValue(true)
 		}
 
 		tlsContext = &envoy_tls_v3.DownstreamTlsContext{
@@ -302,7 +316,7 @@ func xdsNewTransportSocket(
 		}
 	}
 
-	any, err := ptypes.MarshalAny(tlsContext)
+	any, err := anypb.New(tlsContext)
 	require.NoError(t, err)
 
 	return &envoy_core_v3.TransportSocket{
@@ -327,6 +341,18 @@ func xdsNewFilter(t *testing.T, name string, cfg proto.Message) *envoy_listener_
 	return f
 }
 
+func xdsNewListenerFilter(t *testing.T, name string, cfg proto.Message) *envoy_listener_v3.ListenerFilter {
+	f, err := makeEnvoyListenerFilter(name, cfg)
+	require.NoError(t, err)
+	return f
+}
+
+func xdsNewHttpFilter(t *testing.T, name string, cfg proto.Message) *envoy_http_v3.HttpFilter {
+	f, err := makeEnvoyHTTPFilter(name, cfg)
+	require.NoError(t, err)
+	return f
+}
+
 func mustHashResource(t *testing.T, res proto.Message) string {
 	v, err := hashResource(res)
 	require.NoError(t, err)
@@ -349,11 +375,11 @@ func makeTestResource(t *testing.T, raw interface{}) *envoy_discovery_v3.Resourc
 		}
 	case proto.Message:
 
-		any, err := ptypes.MarshalAny(res)
+		any, err := anypb.New(res)
 		require.NoError(t, err)
 
 		return &envoy_discovery_v3.Resource{
-			Name:     getResourceName(res),
+			Name:     xdscommon.GetResourceName(res),
 			Version:  mustHashResource(t, res),
 			Resource: any,
 		}
@@ -371,22 +397,22 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			Namespace:  "default",
 			Datacenter: "dc1",
 			Service:    "db",
-		}
+		}.URI().String()
 
 		geocacheSNI  = "geo-cache.default.dc1.query.11111111-2222-3333-4444-555555555555.consul"
-		geocacheURIs = []connect.SpiffeIDService{
-			{
+		geocacheURIs = []string{
+			connect.SpiffeIDService{
 				Host:       "11111111-2222-3333-4444-555555555555.consul",
 				Namespace:  "default",
 				Datacenter: "dc1",
 				Service:    "geo-cache-target",
-			},
-			{
+			}.URI().String(),
+			connect.SpiffeIDService{
 				Host:       "11111111-2222-3333-4444-555555555555.consul",
 				Namespace:  "default",
 				Datacenter: "dc2",
 				Service:    "geo-cache-target",
-			},
+			}.URI().String(),
 		}
 	)
 
@@ -397,7 +423,7 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			ClusterDiscoveryType: &envoy_cluster_v3.Cluster_Type{
 				Type: envoy_cluster_v3.Cluster_STATIC,
 			},
-			ConnectTimeout: ptypes.DurationProto(5 * time.Second),
+			ConnectTimeout: durationpb.New(5 * time.Second),
 			LoadAssignment: &envoy_endpoint_v3.ClusterLoadAssignment{
 				ClusterName: "local_app",
 				Endpoints: []*envoy_endpoint_v3.LocalityLbEndpoints{{
@@ -422,7 +448,7 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			CommonLbConfig: &envoy_cluster_v3.Cluster_CommonLbConfig{
 				HealthyPanicThreshold: &envoy_type_v3.Percent{Value: 0},
 			},
-			ConnectTimeout:  ptypes.DurationProto(5 * time.Second),
+			ConnectTimeout:  durationpb.New(5 * time.Second),
 			TransportSocket: xdsNewUpstreamTransportSocket(t, snap, dbSNI, dbURI),
 		}
 	case "tcp:db:timeout":
@@ -440,11 +466,11 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			CommonLbConfig: &envoy_cluster_v3.Cluster_CommonLbConfig{
 				HealthyPanicThreshold: &envoy_type_v3.Percent{Value: 0},
 			},
-			ConnectTimeout:  ptypes.DurationProto(1337 * time.Second),
+			ConnectTimeout:  durationpb.New(1337 * time.Second),
 			TransportSocket: xdsNewUpstreamTransportSocket(t, snap, dbSNI, dbURI),
 		}
 	case "http2:db":
-		return &envoy_cluster_v3.Cluster{
+		c := &envoy_cluster_v3.Cluster{
 			Name: dbSNI,
 			ClusterDiscoveryType: &envoy_cluster_v3.Cluster_Type{
 				Type: envoy_cluster_v3.Cluster_EDS,
@@ -458,10 +484,24 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			CommonLbConfig: &envoy_cluster_v3.Cluster_CommonLbConfig{
 				HealthyPanicThreshold: &envoy_type_v3.Percent{Value: 0},
 			},
-			ConnectTimeout:       ptypes.DurationProto(5 * time.Second),
-			TransportSocket:      xdsNewUpstreamTransportSocket(t, snap, dbSNI, dbURI),
-			Http2ProtocolOptions: &envoy_core_v3.Http2ProtocolOptions{},
+			ConnectTimeout:  durationpb.New(5 * time.Second),
+			TransportSocket: xdsNewUpstreamTransportSocket(t, snap, dbSNI, dbURI),
 		}
+		typedExtensionProtocolOptions := &envoy_upstreams_v3.HttpProtocolOptions{
+			UpstreamProtocolOptions: &envoy_upstreams_v3.HttpProtocolOptions_ExplicitHttpConfig_{
+				ExplicitHttpConfig: &envoy_upstreams_v3.HttpProtocolOptions_ExplicitHttpConfig{
+					ProtocolConfig: &envoy_upstreams_v3.HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions{
+						Http2ProtocolOptions: &envoy_core_v3.Http2ProtocolOptions{},
+					},
+				},
+			},
+		}
+		typedExtensionProtocolOptionsEncoded, err := anypb.New(typedExtensionProtocolOptions)
+		require.NoError(t, err)
+		c.TypedExtensionProtocolOptions = map[string]*anypb.Any{
+			"envoy.extensions.upstreams.http.v3.HttpProtocolOptions": typedExtensionProtocolOptionsEncoded,
+		}
+		return c
 	case "http:db":
 		return &envoy_cluster_v3.Cluster{
 			Name: dbSNI,
@@ -477,7 +517,7 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			CommonLbConfig: &envoy_cluster_v3.Cluster_CommonLbConfig{
 				HealthyPanicThreshold: &envoy_type_v3.Percent{Value: 0},
 			},
-			ConnectTimeout:  ptypes.DurationProto(5 * time.Second),
+			ConnectTimeout:  durationpb.New(5 * time.Second),
 			TransportSocket: xdsNewUpstreamTransportSocket(t, snap, dbSNI, dbURI),
 			// HttpProtocolOptions: &envoy_core_v3.Http1ProtocolOptions{},
 		}
@@ -492,7 +532,7 @@ func makeTestCluster(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName st
 			},
 			CircuitBreakers:  &envoy_cluster_v3.CircuitBreakers{},
 			OutlierDetection: &envoy_cluster_v3.OutlierDetection{},
-			ConnectTimeout:   ptypes.DurationProto(5 * time.Second),
+			ConnectTimeout:   durationpb.New(5 * time.Second),
 			TransportSocket:  xdsNewUpstreamTransportSocket(t, snap, geocacheSNI, geocacheURIs...),
 		}
 	default:
@@ -558,10 +598,34 @@ func makeTestEndpoints(t *testing.T, _ *proxycfg.ConfigSnapshot, fixtureName str
 
 func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName string) *envoy_listener_v3.Listener {
 	switch fixtureName {
+	case "tcp:bad_public_listener":
+		return &envoy_listener_v3.Listener{
+			// Envoy can't bind to port 1
+			Name:             "public_listener:0.0.0.0:1",
+			Address:          response.MakeAddress("0.0.0.0", 1),
+			TrafficDirection: envoy_core_v3.TrafficDirection_INBOUND,
+			FilterChains: []*envoy_listener_v3.FilterChain{
+				{
+					TransportSocket: xdsNewPublicTransportSocket(t, snap),
+					Filters: []*envoy_listener_v3.Filter{
+						xdsNewFilter(t, "envoy.filters.network.rbac", &envoy_network_rbac_v3.RBAC{
+							Rules:      &envoy_rbac_v3.RBAC{},
+							StatPrefix: "connect_authz",
+						}),
+						xdsNewFilter(t, "envoy.filters.network.tcp_proxy", &envoy_tcp_proxy_v3.TcpProxy{
+							ClusterSpecifier: &envoy_tcp_proxy_v3.TcpProxy_Cluster{
+								Cluster: "local_app",
+							},
+							StatPrefix: "public_listener",
+						}),
+					},
+				},
+			},
+		}
 	case "tcp:public_listener":
 		return &envoy_listener_v3.Listener{
 			Name:             "public_listener:0.0.0.0:9999",
-			Address:          makeAddress("0.0.0.0", 9999),
+			Address:          response.MakeAddress("0.0.0.0", 9999),
 			TrafficDirection: envoy_core_v3.TrafficDirection_INBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
@@ -584,7 +648,7 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 	case "tcp:db":
 		return &envoy_listener_v3.Listener{
 			Name:             "db:127.0.0.1:9191",
-			Address:          makeAddress("127.0.0.1", 9191),
+			Address:          response.MakeAddress("127.0.0.1", 9191),
 			TrafficDirection: envoy_core_v3.TrafficDirection_OUTBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
@@ -593,7 +657,7 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 							ClusterSpecifier: &envoy_tcp_proxy_v3.TcpProxy_Cluster{
 								Cluster: "db.default.dc1.internal.11111111-2222-3333-4444-555555555555.consul",
 							},
-							StatPrefix: "upstream.db.default.dc1",
+							StatPrefix: "upstream.db.default.default.dc1",
 						}),
 					},
 				},
@@ -602,22 +666,25 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 	case "http2:db":
 		return &envoy_listener_v3.Listener{
 			Name:             "db:127.0.0.1:9191",
-			Address:          makeAddress("127.0.0.1", 9191),
+			Address:          response.MakeAddress("127.0.0.1", 9191),
 			TrafficDirection: envoy_core_v3.TrafficDirection_OUTBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
 					Filters: []*envoy_listener_v3.Filter{
 						xdsNewFilter(t, "envoy.filters.network.http_connection_manager", &envoy_http_v3.HttpConnectionManager{
 							HttpFilters: []*envoy_http_v3.HttpFilter{
-								{Name: "envoy.filters.http.router"},
+								xdsNewHttpFilter(t, "envoy.filters.http.router", &envoy_http_router_v3.Router{}),
 							},
 							RouteSpecifier: &envoy_http_v3.HttpConnectionManager_RouteConfig{
 								RouteConfig: makeTestRoute(t, "http2:db:inline"),
 							},
-							StatPrefix: "upstream.db.default.dc1",
+							StatPrefix: "upstream.db.default.default.dc1",
 							Tracing: &envoy_http_v3.HttpConnectionManager_Tracing{
 								RandomSampling: &envoy_type_v3.Percent{Value: 0},
 							},
+							UpgradeConfigs: []*envoy_http_v3.HttpConnectionManager_UpgradeConfig{
+								{UpgradeType: "websocket"},
+							},
 							Http2ProtocolOptions: &envoy_core_v3.Http2ProtocolOptions{},
 						}),
 					},
@@ -627,14 +694,14 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 	case "http2:db:rds":
 		return &envoy_listener_v3.Listener{
 			Name:             "db:127.0.0.1:9191",
-			Address:          makeAddress("127.0.0.1", 9191),
+			Address:          response.MakeAddress("127.0.0.1", 9191),
 			TrafficDirection: envoy_core_v3.TrafficDirection_OUTBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
 					Filters: []*envoy_listener_v3.Filter{
 						xdsNewFilter(t, "envoy.filters.network.http_connection_manager", &envoy_http_v3.HttpConnectionManager{
 							HttpFilters: []*envoy_http_v3.HttpFilter{
-								{Name: "envoy.filters.http.router"},
+								xdsNewHttpFilter(t, "envoy.filters.http.router", &envoy_http_router_v3.Router{}),
 							},
 							RouteSpecifier: &envoy_http_v3.HttpConnectionManager_Rds{
 								Rds: &envoy_http_v3.Rds{
@@ -642,10 +709,13 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 									ConfigSource:    xdsNewADSConfig(),
 								},
 							},
-							StatPrefix: "upstream.db.default.dc1",
+							StatPrefix: "upstream.db.default.default.dc1",
 							Tracing: &envoy_http_v3.HttpConnectionManager_Tracing{
 								RandomSampling: &envoy_type_v3.Percent{Value: 0},
 							},
+							UpgradeConfigs: []*envoy_http_v3.HttpConnectionManager_UpgradeConfig{
+								{UpgradeType: "websocket"},
+							},
 							Http2ProtocolOptions: &envoy_core_v3.Http2ProtocolOptions{},
 						}),
 					},
@@ -655,14 +725,14 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 	case "http:db:rds":
 		return &envoy_listener_v3.Listener{
 			Name:             "db:127.0.0.1:9191",
-			Address:          makeAddress("127.0.0.1", 9191),
+			Address:          response.MakeAddress("127.0.0.1", 9191),
 			TrafficDirection: envoy_core_v3.TrafficDirection_OUTBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
 					Filters: []*envoy_listener_v3.Filter{
 						xdsNewFilter(t, "envoy.filters.network.http_connection_manager", &envoy_http_v3.HttpConnectionManager{
 							HttpFilters: []*envoy_http_v3.HttpFilter{
-								{Name: "envoy.filters.http.router"},
+								xdsNewHttpFilter(t, "envoy.filters.http.router", &envoy_http_router_v3.Router{}),
 							},
 							RouteSpecifier: &envoy_http_v3.HttpConnectionManager_Rds{
 								Rds: &envoy_http_v3.Rds{
@@ -670,10 +740,13 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 									ConfigSource:    xdsNewADSConfig(),
 								},
 							},
-							StatPrefix: "upstream.db.default.dc1",
+							StatPrefix: "upstream.db.default.default.dc1",
 							Tracing: &envoy_http_v3.HttpConnectionManager_Tracing{
 								RandomSampling: &envoy_type_v3.Percent{Value: 0},
 							},
+							UpgradeConfigs: []*envoy_http_v3.HttpConnectionManager_UpgradeConfig{
+								{UpgradeType: "websocket"},
+							},
 							// HttpProtocolOptions: &envoy_core_v3.Http1ProtocolOptions{},
 						}),
 					},
@@ -683,7 +756,7 @@ func makeTestListener(t *testing.T, snap *proxycfg.ConfigSnapshot, fixtureName s
 	case "tcp:geo-cache":
 		return &envoy_listener_v3.Listener{
 			Name:             "prepared_query:geo-cache:127.10.10.10:8181",
-			Address:          makeAddress("127.10.10.10", 8181),
+			Address:          response.MakeAddress("127.10.10.10", 8181),
 			TrafficDirection: envoy_core_v3.TrafficDirection_OUTBOUND,
 			FilterChains: []*envoy_listener_v3.FilterChain{
 				{
@@ -709,7 +782,7 @@ func makeTestRoute(t *testing.T, fixtureName string) *envoy_route_v3.RouteConfig
 	case "http2:db", "http:db":
 		return &envoy_route_v3.RouteConfiguration{
 			Name:             "db",
-			ValidateClusters: makeBoolValue(true),
+			ValidateClusters: response.MakeBoolValue(true),
 			VirtualHosts: []*envoy_route_v3.VirtualHost{
 				{
 					Name:    "db",
@@ -738,7 +811,7 @@ func makeTestRoute(t *testing.T, fixtureName string) *envoy_route_v3.RouteConfig
 			Name: "db",
 			VirtualHosts: []*envoy_route_v3.VirtualHost{
 				{
-					Name:    "db.default.dc1",
+					Name:    "db.default.default.dc1",
 					Domains: []string{"*"},
 					Routes: []*envoy_route_v3.Route{
 						{
@@ -765,13 +838,6 @@ func makeTestRoute(t *testing.T, fixtureName string) *envoy_route_v3.RouteConfig
 	}
 }
 
-func runStep(t *testing.T, name string, fn func(t *testing.T)) {
-	t.Helper()
-	if !t.Run(name, fn) {
-		t.FailNow()
-	}
-}
-
 func requireProtocolVersionGauge(
 	t *testing.T,
 	scenario *testServerScenario,
@@ -782,7 +848,7 @@ func requireProtocolVersionGauge(
 	require.Len(t, data, 1)
 
 	item := data[0]
-	require.Len(t, item.Gauges, 1)
+	require.Len(t, item.Gauges, 2)
 
 	val, ok := item.Gauges["consul.xds.test.xds.server.streams;version="+xdsVersion]
 	require.True(t, ok)
