@@ -1,3 +1,8 @@
+/**
+ * Copyright (c) HashiCorp, Inc.
+ * SPDX-License-Identifier: BUSL-1.1
+ */
+
 import Component from '@ember/component';
 
 export default Component.extend({
@@ -5,20 +10,24 @@ export default Component.extend({
 
   shouldShowPermissionForm: false,
 
+  openModal() {
+    this.modal?.open();
+  },
+
   actions: {
-    createNewLabel: function(template, term) {
+    createNewLabel: function (template, term) {
       return template.replace(/{{term}}/g, term);
     },
-    isUnique: function(items, term) {
+    isUnique: function (items, term) {
       return !items.findBy('Name', term);
     },
-    add: function(name, changeset, value) {
-      if (!(changeset.get(name) || []).includes(value) && value.isNew) {
+    add: function (name, changeset, value) {
+      if (!(changeset.get(name) || []).includes(value)) {
         changeset.pushObject(name, value);
         changeset.validate();
       }
     },
-    delete: function(name, changeset, value) {
+    delete: function (name, changeset, value) {
       if ((changeset.get(name) || []).includes(value)) {
         changeset.removeObject(name, value);
         changeset.validate();
